﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ProjectAircraftCarrier.DrawingObjects;
using ProjectAircraftCarrier.Entities;

namespace ProjectAircraftCarrier.DrawingObjects
{
    /// <summary>
    /// Класс, отвечающий за прорисовку и перемещение объекта-сущности
    /// </summary>
    internal class DrawingAircraftCarrier : DrawingWarship
    {
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес</param>
        /// <param name="bodyColor">Основной цвет</param>
        /// <param name="additionalColor">Дополнительный цвет</param>
        /// <param name="runway">Признак наличия антикрыла</param>
        /// <param name="cabin">Признак наличия гоночной полосы</param>
        /// <param name="width">Ширина картинки</param>
        /// <param name="height">Высота картинки</param>
        public DrawingAircraftCarrier(int speed, double weight, Color bodyColor, Color additionalColor, bool runway, bool cabin, int width, int height) :
            base(speed, weight, bodyColor, width, height, 196, 58)
        {
            if (EntityWarship != null)
            {
                EntityWarship = new EntityAircraftCarrier(speed, weight, bodyColor, additionalColor, runway, cabin);
            }
        }

        public override void DrawTransport(Graphics g)
        {
            if (EntityWarship is not EntityAircraftCarrier aircraftCarrier)
            {
                return;
            }
            Pen pen = new(Color.Black);
            Brush additionalBrush = new
            SolidBrush(aircraftCarrier.AdditionalColor);

            base.DrawTransport(g);
            //впп
            if (aircraftCarrier.Runway)
            {
                Pen penWhite = new Pen(Color.White, 3);
                Pen penThinWhite = new Pen(Color.White, 1);
                g.DrawLine(penWhite, _startPosX + 4, _startPosY + 14, _startPosX + 114, _startPosY + 1);
                g.DrawLine(penWhite, _startPosX + 4, _startPosY + 26, _startPosX + 156, _startPosY + 8);
                g.DrawLine(penThinWhite, _startPosX + 3, _startPosY + 20, _startPosX + 148, _startPosY + 3);
            }
            // кабина
            if (aircraftCarrier.Cabin)
            {
                Point[] PointCabin = {
                new Point(_startPosX + 15, _startPosY + 42),
                new Point(_startPosX + 17, _startPosY + 41),
                new Point(_startPosX + 22, _startPosY + 41),
                new Point(_startPosX + 22, _startPosY + 38),
                new Point(_startPosX + 25, _startPosY + 38),
                new Point(_startPosX + 25, _startPosY + 41),
                new Point(_startPosX + 35, _startPosY + 41),
                new Point(_startPosX + 35, _startPosY + 38),
                new Point(_startPosX + 38, _startPosY + 38),
                new Point(_startPosX + 38, _startPosY + 41),
                new Point(_startPosX + 43, _startPosY + 41),
                new Point(_startPosX + 43, _startPosY + 38),
                new Point(_startPosX + 46, _startPosY + 38),
                new Point(_startPosX + 46, _startPosY + 41),
                new Point(_startPosX + 51, _startPosY + 41),
                new Point(_startPosX + 52, _startPosY + 42),
                new Point(_startPosX + 52, _startPosY + 45),
                new Point(_startPosX + 51, _startPosY + 46),
                new Point(_startPosX + 46, _startPosY + 46),
                new Point(_startPosX + 46, _startPosY + 49),
                new Point(_startPosX + 43, _startPosY + 49),
                new Point(_startPosX + 43, _startPosY + 46),
                new Point(_startPosX + 38, _startPosY + 46),
                new Point(_startPosX + 38, _startPosY + 49),
                new Point(_startPosX + 35, _startPosY + 49),
                new Point(_startPosX + 35, _startPosY + 46),
                new Point(_startPosX + 25, _startPosY + 46),
                new Point(_startPosX + 25, _startPosY + 49),
                new Point(_startPosX + 22, _startPosY + 49),
                new Point(_startPosX + 22, _startPosY + 46),
                new Point(_startPosX + 17, _startPosY + 46),
                new Point(_startPosX + 15, _startPosY + 45),
                };
                g.FillPolygon(additionalBrush, PointCabin);
                g.DrawPolygon(pen, PointCabin);
            }
        }
    }
}
