﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ProjectAircraftCarrier.Entities;
using ProjectAircraftCarrier.MovementStrategy;

namespace ProjectAircraftCarrier.DrawingObjects
{
    public class DrawingWarship
    {
        /// <summary>
        /// Класс-сущность
        /// </summary>
        public EntityWarship? EntityWarship { get; protected set; }
        /// <summary>
        /// Ширина окна
        /// </summary>
        private int _pictureWidth;
        /// <summary>
        /// Высота окна
        /// </summary>
        private int _pictureHeight;
        /// <summary>
        /// Левая координата прорисовки авианосца
        /// </summary>
        protected int _startPosX;
        /// <summary>
        /// Верхняя кооридната прорисовки авианосца
        /// </summary>
        protected int _startPosY;
        /// <summary>
        /// Ширина прорисовки авианосца
        /// </summary>
        private readonly int _warshipWidth = 196;
        /// <summary>
        /// Высота прорисовки авианосца
        /// </summary>
        private readonly int _warshipHeight = 58;
        /// <summary>
        /// Координата X объекта
        /// </summary>
        public int GetPosX => _startPosX;
        /// <summary>
        /// Координата Y объекта
        /// </summary>
        public int GetPosY => _startPosY;
        /// <summary>
        /// Ширина объекта
        /// </summary>
        public int GetWidth => _warshipWidth;
        /// <summary>
        /// Высота объекта
        /// </summary>
        public int GetHeight => _warshipHeight;
        /// <summary>
        ///  Конструктор
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес</param>
        /// <param name="bodyColor">Цвет кузова</param>
        /// <param name="width">Ширина картинки</param>
        /// <param name="height">Высота картинки</param>
        /// <returns>true - объект создан, false - проверка не пройдена,нельзя создать объект в этих размерах</returns>
        public DrawingWarship(int speed, double weight, Color bodyColor, int width, int height)
        {
            if (_warshipWidth > width || _warshipHeight > height)
                return;
            _pictureWidth = width;
            _pictureHeight = height;
            EntityWarship = new EntityWarship(speed, weight, bodyColor);
        }
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес</param>
        /// <param name="bodyColor">Основной цвет</param>
        /// <param name="width">Ширина картинки</param>
        /// <param name="height">Высота картинки</param>
        /// <param name="warshipWidth">Ширина прорисовки автомобиля</param>
        /// <param name="warshipHeight">Высота прорисовки автомобиля</param>
        protected DrawingWarship(int speed, double weight, Color bodyColor, int
width, int height, int warshipWidth, int warshipHeight)
        {
            if (_warshipWidth > width || _warshipHeight > height)
                return;
            _pictureWidth = width;
            _pictureHeight = height;
            _warshipWidth = warshipWidth;
            _warshipHeight = warshipHeight;
            EntityWarship = new EntityWarship(speed, weight, bodyColor);
        }
        /// <summary>
        /// Установка позиции
        /// </summary>
        /// <param name="x">Координата X</param>
        /// <param name="y">Координата Y</param>
        public void SetPosition(int x, int y)
        {
            _startPosX = x;
            _startPosY = y;
            if (_startPosX < 0 || _startPosY < 0)
            {
                _startPosX = 50;
                _startPosY = 50;
            }
        }
        /// <summary>
        /// Проверка, что объект может переместится по указанному направлению
        /// </summary>
        /// <param name="direction">Направление</param>
        /// <returns>true - можно переместится по указанному направлению</returns>
        public bool CanMove(DirectionType direction)
        {
            if (EntityWarship == null)
            {
                return false;
            }
            return direction switch
            {
                //влево
                DirectionType.Left => _startPosX - EntityWarship.Step > 0,
                //вверх
                DirectionType.Up => _startPosY - EntityWarship.Step > 0,
                // вправо
                DirectionType.Right => _startPosX + EntityWarship.Step + _warshipWidth < _pictureWidth,
                //вниз
                DirectionType.Down => _startPosY + EntityWarship.Step + _warshipHeight < _pictureHeight,
                _ => false,
            };
        }

        /// <summary>
        /// Изменение направления перемещения
        /// </summary>
        /// <param name="direction">Направление</param>
        public void MoveTransport(DirectionType direction)
        {
            if (!CanMove(direction) || EntityWarship == null)
            {
                return;
            }
            switch (direction)
            {
                //влево
                case DirectionType.Left:
                    _startPosX -= (int)EntityWarship.Step;
                    break;
                //вверх
                case DirectionType.Up:
                    _startPosY -= (int)EntityWarship.Step;
                    break;
                // вправо
                case DirectionType.Right:
                    _startPosX += (int)EntityWarship.Step;
                    break;
                //вниз
                case DirectionType.Down:
                    _startPosY += (int)EntityWarship.Step;
                    break;
            }
        }

        /// <summary>
        /// Прорисовка объекта
        /// </summary>
        /// <param name="g"></param>
        public virtual void DrawTransport(Graphics g)
        {
            if (EntityWarship == null)
            {
                return;
            }
            Pen pen = new(Color.Black);
            Brush br = new SolidBrush(EntityWarship.BodyColor);
            //двигатели
            Point[] PointEngine1 = {
                new Point(_startPosX + 0, _startPosY + 11),
                new Point(_startPosX + 2, _startPosY + 11),
                new Point(_startPosX + 2, _startPosY + 24),
                new Point(_startPosX + 0, _startPosY + 24),
            };
            Point[] PointEngine2 = {
                new Point(_startPosX + 0, _startPosY + 31),
                new Point(_startPosX + 2, _startPosY + 31),
                new Point(_startPosX + 2, _startPosY + 44),
                new Point(_startPosX + 0, _startPosY + 44),
            };
            Brush brBlack = new SolidBrush(Color.Black);
            g.FillPolygon(brBlack, PointEngine1);
            g.DrawPolygon(pen, PointEngine1);
            g.FillPolygon(brBlack, PointEngine2);
            g.DrawPolygon(pen, PointEngine2);
            //границы авианосца
            Point[] PointBorders = {
                new Point(_startPosX + 4, _startPosY + 0),
                new Point(_startPosX + 142, _startPosY + 0),
                new Point(_startPosX + 196, _startPosY + 29),
                new Point(_startPosX + 142, _startPosY + 58),
                new Point(_startPosX + 4, _startPosY + 58),
            };
            Brush brGray = new SolidBrush(Color.Gray);
            g.FillPolygon(br, PointBorders);
            g.DrawPolygon(pen, PointBorders);
            //отсеки
            g.DrawRectangle(pen, _startPosX + 98, _startPosY + 18, 10, 23);
            g.FillRectangle(brBlack, _startPosX + 99, _startPosY + 19, 9, 22);
            g.DrawRectangle(pen, _startPosX + 78, _startPosY + 23, 20, 13);
            g.FillRectangle(brBlack, _startPosX + 79, _startPosY + 24, 19, 12);
            g.DrawEllipse(pen, _startPosX + 138, _startPosY + 22, 11, 11);
            g.FillEllipse(brBlack, _startPosX + 138, _startPosY + 22, 11, 11);
        }
        /// <summary>
        /// Получение объекта IMoveableObject из объекта DrawningCar
        /// </summary>
        public IMoveableObject GetMoveableObject => new DrawingObjectWarship(this);
    }
}