﻿using ProjectAircraftCarrier.Entities;

namespace ProjectAircraftCarrier.DrawingObjects
{
    /// <summary>
    /// Расширение для класса EntityWarship
    /// </summary>
    public static class ExtentionDrawingWarship
    {
        /// <summary>
        /// Создание объекта из строки
        /// </summary>
        /// <param name="info">Строка с данными для создания объекта</param>
        /// <param name="separatorForObject">Разделитель даннных</param>
        /// <param name="width">Ширина</param>
        /// <param name="height">Высота</param>
        /// <returns>Объект</returns>
        public static DrawingWarship? CreateDrawingWarship(this string info,
            char separatorForObject, int width, int height)
        {
            string[] strs = info.Split(separatorForObject);
            if (strs.Length == 3)
            {
                return new DrawingWarship(Convert.ToInt32(strs[0]),
                    Convert.ToInt32(strs[1]), Color.FromName(strs[2]),
                    width, height);
            }
            if (strs.Length == 6)
            {
                return new DrawingAircraftCarrier(Convert.ToInt32(strs[0]),
                    Convert.ToInt32(strs[1]), Color.FromName(strs[2]),
                    Color.FromName(strs[3]), Convert.ToBoolean(strs[4]),
                    Convert.ToBoolean(strs[5]), width, height);
            }
            return null;
        }
        /// <summary>
        /// Получение данных для сохранения в файл
        /// </summary>
        /// <param name="drawingWarship">Сохраняемый объект</param>
        /// <param name="separatorForObject">Разделитель даннных</param>
        /// <returns>Строка с данными по объекту</returns>
        public static string GetDataForSave(this DrawingWarship drawingWarship,
        char separatorForObject)
        {
            var warship = drawingWarship.EntityWarship;
            if (warship == null)
            {
                return string.Empty;
            }
            var str = $"{warship.Speed}{separatorForObject}{warship.Weight}" +
                $"{separatorForObject}{warship.BodyColor.Name}";
            if (warship is not EntityAircraftCarrier aircraftCarrier)
            {
                return str;
            }
            return $"{str}{separatorForObject}{aircraftCarrier.AdditionalColor.Name}{separatorForObject}{aircraftCarrier.Runway}{separatorForObject}{aircraftCarrier.Cabin}";
        }
    }
}

