using ProjectAircraftCarrier.MovementStrategy;
using ProjectAircraftCarrier.DrawingObjects;
using System.Windows.Forms;

namespace ProjectAircraftCarrier
{
    public partial class FormAircraftCarrier : Form
    {
        /// <summary>
        /// -   
        /// </summary>
        private DrawingWarship? _drawingWarship;
        /// <summary>
        ///  
        /// </summary>
        private AbstractStrategy? _abstractStrategy;
        /// <summary>
        ///  
        /// </summary>
        public DrawingWarship? SelectedWarship { get; private set; }
        /// <summary>
        ///  
        /// </summary>
        public FormAircraftCarrier()
        {
            InitializeComponent();
            SelectedWarship = null;
            _abstractStrategy = null;
        }
        /// <summary>
        ///   
        /// </summary>
        private void Draw()
        {
            if (_drawingWarship == null)
            {   
                return;
            }
            Bitmap bmp = new(pictureBoxAircraftCarrier.Width, pictureBoxAircraftCarrier.Height);
            Graphics gr = Graphics.FromImage(bmp);
            _drawingWarship.DrawTransport(gr);
            pictureBoxAircraftCarrier.Image = bmp;

        }
        /// <summary>
        ///    ""
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonCreateAircraftCarrier_Click(object sender, EventArgs e)
        {
            Random random = new();
            Color bodyColor = Color.FromArgb(random.Next(0, 256),
            random.Next(0, 256), random.Next(0, 256));
            ColorDialog dialog = new();
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                bodyColor = dialog.Color;
            }
            Color additColor = Color.FromArgb(random.Next(0, 256),
            random.Next(0, 256), random.Next(0, 256));
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                additColor = dialog.Color;
            }
            _drawingWarship = new DrawingAircraftCarrier(random.Next(100, 300),
            random.Next(1000, 3000), bodyColor,
            additColor, Convert.ToBoolean(random.Next(0, 2)),
            Convert.ToBoolean(random.Next(0, 2)),
            pictureBoxAircraftCarrier.Width, pictureBoxAircraftCarrier.Height);
            _drawingWarship.SetPosition(random.Next(10, 100), random.Next(10, 100));
            Draw();
        }
        /// <summary>
        ///    "Create Warship"
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonCreateWarship_Click(object sender, EventArgs e)
        {
            Random random = new();
            Color color = Color.FromArgb(random.Next(0, 256),
            random.Next(0, 256), random.Next(0, 256));
            ColorDialog dialog = new();
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                color = dialog.Color;
            }
            _drawingWarship = new DrawingWarship(random.Next(100, 300),
            random.Next(1000, 3000), color,
            pictureBoxAircraftCarrier.Width, pictureBoxAircraftCarrier.Height);
            _drawingWarship.SetPosition(random.Next(10, 100), random.Next(10, 100));
            Draw();
        }
        /// <summary>
        ///    
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonMove_Click(object sender, EventArgs e)
        {
            if (_drawingWarship == null)
            {
                return;
            }
            string name = ((Button)sender)?.Name ?? string.Empty;
            switch (name)
            {
                case "buttonUp":
                    _drawingWarship.MoveTransport(DirectionType.Up);
                    break;
                case "buttonDown":
                    _drawingWarship.MoveTransport(DirectionType.Down);
                    break;
                case "buttonLeft":
                    _drawingWarship.MoveTransport(DirectionType.Left);
                    break;
                case "buttonRight":
                    _drawingWarship.MoveTransport(DirectionType.Right);
                    break;
            }
            Draw();
        }
        private void ButtonSelectWarhip_Click(object sender, EventArgs e)
        {
            SelectedWarship = _drawingWarship;
            DialogResult = DialogResult.OK;
        }
        /// <summary>
        ///    ""
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonStep_Click(object sender, EventArgs e)
        {
            if (_drawingWarship == null)
            {
                return;
            }
            if (comboBoxStrategy.Enabled)
            {
                _abstractStrategy = comboBoxStrategy.SelectedIndex switch
                {
                    0 => new MoveToCenter(),
                    1 => new MoveToBorder(),
                    _ => null,
                };
                if (_abstractStrategy == null)
                {
                    return;
                }
                _abstractStrategy.SetData(new DrawingObjectWarship(_drawingWarship), pictureBoxAircraftCarrier.Width, pictureBoxAircraftCarrier.Height);
                comboBoxStrategy.Enabled = false;
            }
            if (_abstractStrategy == null)
            {
                return;
            }
            _abstractStrategy.MakeStep();
            Draw();
            if (_abstractStrategy.GetStatus() == Status.Finish)
            {
                comboBoxStrategy.Enabled = true;
                _abstractStrategy = null;
            }
        }
    }
}
