﻿using ProjectAircraftCarrier.DrawingObjects;
using ProjectAircraftCarrier.Entities;

namespace ProjectAircraftCarrier
{
    public partial class FormWarshipConfig : Form
    {
        /// <summary>
        /// Переменная-выбранный военный корабль
        /// </summary>
        DrawingWarship? _warship = null;
        private event Action<DrawingWarship>? EventAddWarship;
        /// <summary>
        /// Конструктор
        /// </summary>
        public FormWarshipConfig()
        {
            InitializeComponent();
            panelBlack.MouseDown += PanelColor_MouseDown;
            panelPurple.MouseDown += PanelColor_MouseDown;
            panelGray.MouseDown += PanelColor_MouseDown;
            panelGreen.MouseDown += PanelColor_MouseDown;
            panelRed.MouseDown += PanelColor_MouseDown;
            panelWhite.MouseDown += PanelColor_MouseDown;
            panelYellow.MouseDown += PanelColor_MouseDown;
            panelBlue.MouseDown += PanelColor_MouseDown;
            buttonCancel.Click += (s, e) => Close();
        }
        /// <summary>
        /// Отрисовать военний корабль
        /// </summary>
        private void DrawWarship()
        {
            Bitmap bmp = new(pictureBoxObject.Width, pictureBoxObject.Height);
            Graphics gr = Graphics.FromImage(bmp);
            _warship?.SetPosition(5, 5);
            _warship?.DrawTransport(gr);
            pictureBoxObject.Image = bmp;
        }
        /// <summary>
        /// Добавление события
        /// </summary>
        /// <param name="ev">Привязанный метод</param>
        public void AddEvent(Action<DrawingWarship>? ev)
        {
            if (EventAddWarship == null)
            {
                EventAddWarship = ev;
            }
            else
            {
                EventAddWarship += ev;
            }
        }
        /// <summary>
        /// Передаем информацию при нажатии на Label
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void LabelObject_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Label)?.DoDragDrop((sender as Label)?.Name, 
                DragDropEffects.Move | DragDropEffects.Copy);
        }
        /// <summary>
        /// Проверка получаемой информации (ее типа на соответствие требуемому)
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PanelObject_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data?.GetDataPresent(DataFormats.Text) ?? false)
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }
        /// <summary>
        /// Действия при приеме перетаскиваемой информации
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PanelObject_DragDrop(object sender, DragEventArgs e)
        {
            switch (e.Data?.GetData(DataFormats.Text).ToString())
            {
                case "labelSimpleObject":
                    _warship = new DrawingWarship(
                        (int)numericUpDownSpeed.Value, 
                        (int)numericUpDownWeight.Value, Color.White,
                        pictureBoxObject.Width, pictureBoxObject.Height);
                    break;
                case "labelModifiedObject":
                    _warship = new DrawingAircraftCarrier(
                        (int)numericUpDownSpeed.Value, 
                        (int)numericUpDownWeight.Value, Color.White, 
                        Color.Black, checkBoxCabin.Checked, 
                        checkBoxRunway.Checked, 
                        pictureBoxObject.Width,pictureBoxObject.Height);
                    break;
            }
            DrawWarship();
        }
        private void PanelColor_MouseDown(object? sender, MouseEventArgs e)
        {
            (sender as Panel)?.DoDragDrop((sender as Panel)?.BackColor,
                DragDropEffects.Move | DragDropEffects.Copy);
        }
        /// <summary>
        /// Добавление военного корабля
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonAdd_Click(object sender, EventArgs e)
        {
            if (_warship == null)
                return;
            EventAddWarship?.Invoke(_warship);
            Close();
        }
        private void LabelColor_DragDrop(object sender, DragEventArgs e)
        {
            if (_warship?.EntityWarship == null)
                return;
            Color bodyColor = (Color)e.Data.GetData(typeof(Color));
            _warship.EntityWarship.ChangeColor(bodyColor);
            DrawWarship();
        }
        private void LabelAddColor_DragDrop(object sender, DragEventArgs e)
        {
            if ((_warship?.EntityWarship == null) || (_warship is
                DrawingAircraftCarrier == false))
                return;
            Color additionalColor = (Color)e.Data.GetData(typeof(Color));
            ((EntityAircraftCarrier)_warship.EntityWarship).ChangeAddColor(additionalColor);
            DrawWarship();
        }
        private void LabelColor_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(typeof(Color)))
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }
    }
}